@@ -2,32 +2,37 @@
 
 
 import copy
+import datetime
 import glob
 import logging
 import os
 import datetime
-from google.cloud import translate_v2
 from urllib.parse import urlparse
 
+# With lazy-loading
+import pytextrank
 import requests
+import spacy
 from bs4 import BeautifulSoup
 # https://preslav.me/2019/01/09/dotenv-files-python/
 from fake_useragent import UserAgent
+from google.cloud import translate_v2
 
 from scraper.urls import extract_domain
 from scraper.video_extractor import VideoExtractor
 from . import image_extractor
 from . import network
-from . import nlp
 from . import settings
 from . import urls
 from .configuration import Configuration
 from .content_extractor import ContentExtractor
 from .document_cleaner import DocumentCleaner
+from .named_entity_recognition import TextRank4Keyword
 from .output_formatter import OutputFormatter
+from .text import get_stopwords
 from .utils import (URLHelper, RawHelper, extend_config,
                     get_available_language_codes, extract_meta_refresh,
-                    parse_date_str, cleanup_text)
+                    parse_date_str, split_words)
 
 __title__ = 'scraper'
 __author__ = 'Lucas Ou-Yang'
@@ -37,7 +42,7 @@ __maintainer__ = "The Stimson Center"
 __maintainer_email = "cooper@pobox.com"
 
 log = logging.getLogger(__name__)
-
+date_format = "%Y-%m-%d"
 
 class ArticleDownloadState(object):
     NOT_STARTED = 0
@@ -111,7 +116,8 @@ class Article(object):
         # List of authors who have published the article, via parse()
         self.authors = []
 
-        self.publish_date = None
+        current_date = datetime.date.today()
+        self.publish_date = current_date.strftime(date_format)
 
         # Summary generated from the article's body txt
         self.summary = ''
@@ -244,12 +250,13 @@ class Article(object):
                 return self.download(input_html, recursion_counter=recursion_counter + 1)
 
         self.set_html(html)
-        self.set_title(title)
+        if title:
+            self.set_title(title)
 
     def parse(self):
         self.throw_if_not_downloaded_verbose()
 
-        self.doc = self.config.get_parser().fromstring(self.html)
+        self.doc = self.config.get_parser().from_string(self.html)
         self.clean_doc = copy.deepcopy(self.doc)
 
         if self.doc is None:
@@ -298,13 +305,12 @@ class Article(object):
         meta_data = self.extractor.get_meta_data(self.clean_doc)
         self.set_meta_data(meta_data)
 
-        if not self.publish_date:
-            self.set_publish_date(self.extractor.get_publishing_date(self.url, self.clean_doc))
+        self.set_publish_date(self.extractor.get_publishing_date(self.url, self.clean_doc))
 
         # Before any computations on the body, clean DOM object
         self.doc = document_cleaner.clean(self.doc)
 
-        self.top_node = self.extractor.calculate_best_node(self.doc)
+        self.top_node = self.extractor.calculate_best_node(self.doc, self.html)
         if self.top_node is not None:
             video_extractor = VideoExtractor(self.config, self.top_node)
             self.set_movies(video_extractor.get_videos())
@@ -408,25 +414,101 @@ class Article(object):
         self.throw_if_not_downloaded_verbose()
         self.throw_if_not_parsed_verbose()
 
-        nlp.load_stopwords(self.config.get_language())
-        text_keyws = list(nlp.keywords(self.text).keys())
-        title_keyws = list(nlp.keywords(self.title).keys())
-        keyws = set(title_keyws)
-        while len(keyws) < self.config.KEYWORD_COUNT and text_keyws:
-            keyws.add(text_keyws.pop(0))
-        keyws = list(keyws)
-        self.set_keywords(keyws)
+        # With Lazy Loading
+        # nlp = MultiLanguage()
+        # nlp = get_lang_class('xx')
 
-        summary_sents = nlp.summarize(title=self.title, text=self.text, max_sents=self.config.MAX_SUMMARY_SENT)
-        summary = '\n'.join(summary_sents)
+        language_code = self.config.get_language()[0:2]
+        # https://spacy.io/usage/models
+        spacy_language_models = {
+            "zh": "zh_core_web_sm",  # Chinese
+            "da": "da_core_news_sm",  # Danish
+            "nl": "nl_core_news_sm",  # Dutch
+            "en": "en_core_web_sm",  # English
+            "fr": "fr_core_news_sm",  # French
+            "de": "de_core_news_sm",  # German
+            "el": "el_core_news_sm",  # Greek
+            "it": "it_core_news_sm",  # Italian
+            "ja": "ja_core_news_sm",  # Japanese
+            "lt": "lt_core_news_sm",  # Lithuanian
+            "nb": "nb_core_news_sm",  # Norwegian Bokmål
+            "pl": "pl_core_news_sm",  # Polish
+            "pt": "pt_core_news_sm",  # Portuguese
+            "ro": "ro_core_news_sm",  # Romanian
+            "es": "es_core_news_sm"  # Spanish
+        }
+        if language_code in spacy_language_models:
+            nlp = spacy.load(spacy_language_models[language_code])
+        else:
+            # https://github.com/huggingface/neuralcoref/issues/117
+            # nlp = spacy.load("xx_ent_wiki_sm", disable = ['ner', 'parser', 'tagger'])
+            nlp = spacy.load("xx_ent_wiki_sm")
+            nlp.add_pipe(nlp.create_pipe('sentencizer'))
+        # use spacy language specific STOP WORDS
+        stopwords = get_stopwords(language_code)
+        # add PyTextRank to the spaCy pipeline
+        tr = pytextrank.TextRank()
+        nlp.add_pipe(tr.PipelineComponent, name="textrank", last=True)
+        # *************************************************************
+        # THIS STEP CAN TAKE A MINUTE OR TWO
+        tr4w = TextRank4Keyword(nlp)
+        tr4w.analyze(self.text.lower(), candidate_pos=['NOUN', 'PROPN'], window_size=4, lower=False, stopwords=stopwords)
+        keywords = list()
+        for k, v in tr4w.get_keywords().items():
+            keywords.append(k)
+        if len(keywords) == 0:
+            keywords = self.xx_keywords(stopwords)
+        self.set_keywords(keywords)
+        summary = ''.join(map(str, tr4w.get_sentences()))
         self.set_summary(summary)
+        # try to get date from raw text if not found in html
+        current_date = datetime.date.today()
+        if self.publish_date == current_date:
+            dates = tr4w.get_dates()
+            if dates:
+                # even if there are multiple dates returned, usually the first date is best to use
+                self.set_publish_date(dates[0])
+
+    def xx_keywords(self, stopwords, count=10):
+        """Get the top `count` keywords and their frequency scores ignores blacklisted
+        words in stopwords, counts the number of occurrences of each word, and
+        sorts them in reverse natural order (so descending) by number of
+        occurrences.
+        """
+        text = split_words(self.text)
+        # of words before removing blacklist words
+        top_keywords = list()
+        if text:
+            # num_words = len(text)
+            text = [x for x in text if x not in stopwords]
+            freq = {}
+            for word in text:
+                if word in freq:
+                    freq[word] += 1
+                else:
+                    freq[word] = 1
+
+            min_size = min(count, len(freq))
+            keywords = sorted(freq.items(),
+                              key=lambda x: (x[1], x[0]),
+                              reverse=True)
+            keywords = keywords[:min_size]
+            index = 1
+            for k, v in keywords:
+                if index > count:
+                    break
+                if not k.isnumeric():
+                    top_keywords.append(k)
+                    index += 1
+
+        return top_keywords
 
     def parse_tables(self, attributes=None):
         if attributes is None:
             attributes = {"class": "wikitable"}
         ua = UserAgent()
         response = requests.get(self.url, headers={'User-Agent': ua.random})
-        soup = BeautifulSoup(response.text, 'lxml')
+        soup = BeautifulSoup(response.text, 'html.parser')
         tables = soup.findAll("table", attributes)
 
         # # show tables
@@ -439,7 +521,7 @@ class Article(object):
         self.tables = list()
         for tn, table in enumerate(tables):
             # preinit list of lists
-            # noinspection PyUnusedLocal
+            # noinspection PyUnusedLocal,PyBroadException
             try:
                 caption = table.find('caption')
                 table_name = caption.get_text().rstrip()
@@ -471,7 +553,7 @@ class Article(object):
                     l = 0
                     for k in range(rspan):
                         # Shifts to the first empty cell of this row
-                        # noinspection PyUnusedLocal
+                        # noinspection PyUnusedLocal,PyBroadException
                         try:
                             while data[i + k][j + l]:
                                 l += 1
@@ -549,7 +631,6 @@ class Article(object):
 
     def set_title(self, input_title):
         if input_title:
-            input_title = cleanup_text(input_title.strip())
             self.title = input_title[:self.config.MAX_TITLE]
 
     def set_text(self, text):
@@ -558,12 +639,12 @@ class Article(object):
             if self.config.translate is True and os.environ.get('GOOGLE_APPLICATION_CREDENTIALS'):
                 translator = translate_v2.Client()
                 raw_dict = translator.translate(text, target_language='en', format_="text")
+                # noinspection PyTypeChecker
                 text = raw_dict['translatedText']
                 text = text.replace("  ", " ")
                 text = text[:self.config.MAX_TEXT]
                 self.config._language = 'en'
             elif self.meta_lang == 'en':
-                # text = cleanup_text(text.strip())
                 text = text.replace("  ", " ")
                 text = text[:self.config.MAX_TEXT]
             self.text = text
@@ -618,20 +699,20 @@ class Article(object):
     def set_authors(self, authors):
         """Authors are in ["firstName lastName", "firstName lastName"] format
         """
-        if not isinstance(authors, list):
-            raise Exception("authors input must be list!")
-        if authors:
-            self.authors = authors[:self.config.MAX_AUTHORS]
-        else:
+        for author in authors:
+            if author and author not in self.authors and len(self.authors) < self.config.MAX_AUTHORS:
+                self.authors.append(author)
+        if not self.authors:
             # if unable to determine article's author, use toplevel domain from url for credits
-            self.authors.append(extract_domain(self.url))
+            domain, subdomain = extract_domain(self.url)
+            self.authors.append(domain)
+        # self.authors = self.authors[:self.config.MAX_AUTHORS]
 
     def set_summary(self, summary):
         """Summary here refers to a paragraph of text from the
         title text and body text
         """
-        self.summary = cleanup_text(summary)
-        self.summary = self.summary[:self.config.MAX_SUMMARY]
+        self.summary = summary[:self.config.MAX_SUMMARY]
 
     def set_meta_language(self, meta_lang):
         """Save langauges in their ISO 2-character form
@@ -671,8 +752,8 @@ class Article(object):
 
     def set_publish_date(self, publish_date):
         if isinstance(publish_date, datetime.datetime):
-            self.publish_date = publish_date.strftime("%Y-%m-%d")
-        else:
+            self.publish_date = publish_date.strftime(date_format)
+        elif publish_date:
             self.publish_date = publish_date
 
     def throw_if_not_downloaded_verbose(self):
@@ -691,21 +772,3 @@ class Article(object):
         """
         if not self.is_parsed:
             raise ArticleException('You must `parse()` an article first!')
-
-
-class ArticleExecutor(Article):
-    def __init__(self, url):
-        config = Configuration()
-        config.memoize_articles = False
-        pdf_defaults = {
-            # "application/pdf": "%PDF-",
-            # "application/x-pdf": "%PDF-",
-            "application/x-bzpdf": "%PDF-",
-            "application/x-gzpdf": "%PDF-"
-        }
-        Article.__init__(self,
-                         url.rstrip(),
-                         request_timeout=config.request_timeout,
-                         ignored_content_types_defaults=pdf_defaults
-                         )
-        self.build()
\ No newline at end of file
