@@ -3,19 +3,22 @@
 All unit tests for the scraper pdf processing should be contained in this file.
 """
 
-from scraper import Article
+from scraper import Article, Configuration
 from tests.conftest import print_test
 
 
 @print_test
 def test_article_pdf_ignoring():
+    config = Configuration()
     empty_pdf = "%PDF-"  # empty PDF constant
-    article = Article(url='http://www.technik-medien.at/ePaper_Download/'
-                          'IoT4Industry+Business_2018-10-31_2018-03.pdf',
-                      ignored_content_types_defaults={"application/pdf": empty_pdf,
-                                                      "application/x-pdf": empty_pdf,
-                                                      "application/x-bzpdf": empty_pdf,
-                                                      "application/x-gzpdf": empty_pdf})
+    config.ignored_content_types_defaults = {"application/pdf": empty_pdf,
+                                             "application/x-pdf": empty_pdf,
+                                             "application/x-bzpdf": empty_pdf,
+                                             "application/x-gzpdf": empty_pdf}
+    article = Article(
+        url='https://www.adobe.com/pdf/pdfs/ISO32000-1PublicPatentLicense.pdf',
+        config=config
+    )
     article.download()
     assert empty_pdf == article.html
 
